@@ -1,5 +1,5 @@
 /*
- * "$Id: emit.c 7224 2008-01-16 23:42:35Z mike $"
+ * "$Id: emit.c 7864 2008-08-26 03:43:28Z mike $"
  *
  *   PPD code emission routines for the Common UNIX Printing System (CUPS).
  *
@@ -18,19 +18,21 @@
  *
  * Contents:
  *
- *   ppdCollect()        - Collect all marked options that reside in the
- *                         specified section.
- *   ppdCollect2()       - Collect all marked options that reside in the
- *                         specified section and minimum order.
- *   ppdEmit()           - Emit code for marked options to a file.
- *   ppdEmitAfterOrder() - Emit a subset of the code for marked options to a
- *                         file.
- *   ppdEmitFd()         - Emit code for marked options to a file.
- *   ppdEmitJCL()        - Emit code for JCL options to a file.
- *   ppdEmitJCLEnd()     - Emit JCLEnd code to a file.
- *   ppdEmitString()     - Get a string containing the code for marked options.
- *   ppd_handle_media()  - Handle media selection...
- *   ppd_sort()          - Sort options by ordering numbers...
+ *   ppdCollect()          - Collect all marked options that reside in the
+ *                           specified section.
+ *   ppdCollect2()         - Collect all marked options that reside in the
+ *                           specified section and minimum order.
+ *   ppdEmit()             - Emit code for marked options to a file.
+ *   ppdEmitAfterOrder()   - Emit a subset of the code for marked options to a
+ *                           file.
+ *   ppdEmitFd()           - Emit code for marked options to a file.
+ *   ppdEmitJCL()          - Emit code for JCL options to a file.
+ *   ppdEmitJCLEnd()       - Emit JCLEnd code to a file.
+ *   ppdEmitString()       - Get a string containing the code for marked
+ *                           options.
+ *   ppd_compare_cparams() - Compare the order of two custom parameters.
+ *   ppd_handle_media()    - Handle media selection...
+ *   ppd_sort()            - Sort options by ordering numbers...
  */
 
 /*
@@ -54,6 +56,7 @@
  * Local functions...
  */
 
+static int	ppd_compare_cparams(ppd_cparam_t *a, ppd_cparam_t *b);
 static void	ppd_handle_media(ppd_file_t *ppd);
 static int	ppd_sort(ppd_choice_t **c1, ppd_choice_t **c2);
 
@@ -475,7 +478,7 @@ ppdEmitJCLEnd(ppd_file_t *ppd,		/* I - PPD file record */
     */
 
     fputs("\033%-12345X@PJL\n", fp);
-    fputs("@PJL RDYMSG DISPLAY = \"READY\"\n", fp);
+    fputs("@PJL RDYMSG DISPLAY = \"\"\n", fp);
     fputs(ppd->jcl_end + 9, fp);
   }
   else
@@ -547,7 +550,40 @@ ppdEmitString(ppd_file_t    *ppd,	/* I - PPD file record */
 
   for (i = 0, bufsize = 1; i < count; i ++)
   {
-    if (section != PPD_ORDER_EXIT && section != PPD_ORDER_JCL)
+    if (section == PPD_ORDER_JCL)
+    {
+      if (!strcasecmp(choices[i]->choice, "Custom") &&
+	  (coption = ppdFindCustomOption(ppd, choices[i]->option->keyword))
+	      != NULL)
+      {
+       /*
+        * Add space to account for custom parameter substitution...
+	*/
+
+        for (cparam = (ppd_cparam_t *)cupsArrayFirst(coption->params);
+	     cparam;
+	     cparam = (ppd_cparam_t *)cupsArrayNext(coption->params))
+	{
+          switch (cparam->type)
+	  {
+	    case PPD_CUSTOM_CURVE :
+	    case PPD_CUSTOM_INVCURVE :
+	    case PPD_CUSTOM_POINTS :
+	    case PPD_CUSTOM_REAL :
+	    case PPD_CUSTOM_INT :
+	        bufsize += 10;
+	        break;
+
+	    case PPD_CUSTOM_PASSCODE :
+	    case PPD_CUSTOM_PASSWORD :
+	    case PPD_CUSTOM_STRING :
+	        bufsize += strlen(cparam->current.custom_string);
+	        break;
+          }
+	}
+      }
+    }
+    else if (section != PPD_ORDER_EXIT)
     {
       bufsize += 3;			/* [{\n */
 
@@ -626,7 +662,89 @@ ppdEmitString(ppd_file_t    *ppd,	/* I - PPD file record */
   */
 
   for (i = 0, bufptr = buffer; i < count; i ++, bufptr += strlen(bufptr))
-    if (section != PPD_ORDER_EXIT && section != PPD_ORDER_JCL)
+    if (section == PPD_ORDER_JCL)
+    {
+      if (!strcasecmp(choices[i]->choice, "Custom") &&
+          (coption = ppdFindCustomOption(ppd, choices[i]->option->keyword))
+	      != NULL)
+      {
+       /*
+        * Handle substitutions in custom JCL options...
+	*/
+
+	char	*cptr;			/* Pointer into code */
+	int	pnum;			/* Parameter number */
+
+
+        for (cptr = choices[i]->code; *cptr && bufptr < bufend;)
+	{
+	  if (*cptr == '\\')
+	  {
+	    cptr ++;
+
+	    if (isdigit(*cptr & 255))
+	    {
+	     /*
+	      * Substitute parameter...
+	      */
+
+              pnum = *cptr++ - '0';
+	      while (isalnum(*cptr & 255))
+	        pnum = pnum * 10 + *cptr - '0';
+
+              for (cparam = (ppd_cparam_t *)cupsArrayFirst(coption->params);
+	           cparam;
+		   cparam = (ppd_cparam_t *)cupsArrayNext(coption->params))
+		if (cparam->order == pnum)
+		  break;
+
+              if (cparam)
+	      {
+	        switch (cparam->type)
+		{
+		  case PPD_CUSTOM_CURVE :
+		  case PPD_CUSTOM_INVCURVE :
+		  case PPD_CUSTOM_POINTS :
+		  case PPD_CUSTOM_REAL :
+		      bufptr = _cupsStrFormatd(bufptr, bufend,
+					       cparam->current.custom_real,
+					       loc);
+		      break;
+
+		  case PPD_CUSTOM_INT :
+		      snprintf(bufptr, bufend - bufptr, "%d",
+		               cparam->current.custom_int);
+		      bufptr += strlen(bufptr);
+		      break;
+
+		  case PPD_CUSTOM_PASSCODE :
+		  case PPD_CUSTOM_PASSWORD :
+		  case PPD_CUSTOM_STRING :
+		      strlcpy(bufptr, cparam->current.custom_string,
+		              bufend - bufptr);
+		      bufptr += strlen(bufptr);
+		      break;
+		}
+	      }
+	    }
+	    else if (*cptr)
+	      *bufptr++ = *cptr++;
+	  }
+	  else
+	    *bufptr++ = *cptr++;
+	}
+      }
+      else
+      {
+       /*
+        * Otherwise just copy the option code directly...
+	*/
+
+        strlcpy(bufptr, choices[i]->code, bufend - bufptr + 1);
+        bufptr += strlen(bufptr);
+      }
+    }
+    else if (section != PPD_ORDER_EXIT)
     {
      /*
       * Add wrapper commands to prevent printer errors for unsupported
@@ -759,8 +877,7 @@ ppdEmitString(ppd_file_t    *ppd,	/* I - PPD file record */
 	}
       }
       else if (!strcasecmp(choices[i]->choice, "Custom") &&
-               (coption = ppdFindCustomOption(ppd,
-	                                      choices[i]->option->keyword))
+               (coption = ppdFindCustomOption(ppd, choices[i]->option->keyword))
 	           != NULL)
       {
        /*
@@ -768,15 +885,23 @@ ppdEmitString(ppd_file_t    *ppd,	/* I - PPD file record */
 	*/
 
         const char	*s;		/* Pointer into string value */
+        cups_array_t	*params;	/* Parameters in the correct output order */
 
 
+        params = cupsArrayNew((cups_array_func_t)ppd_compare_cparams, NULL);
+
+        for (cparam = (ppd_cparam_t *)cupsArrayFirst(coption->params);
+	     cparam;
+	     cparam = (ppd_cparam_t *)cupsArrayNext(coption->params))
+          cupsArrayAdd(params, cparam);
+
         snprintf(bufptr, bufend - bufptr + 1,
 	         "%%%%BeginFeature: *Custom%s True\n", coption->keyword);
         bufptr += strlen(bufptr);
 
-        for (cparam = (ppd_cparam_t *)cupsArrayFirst(coption->params);
+        for (cparam = (ppd_cparam_t *)cupsArrayFirst(params);
 	     cparam;
-	     cparam = (ppd_cparam_t *)cupsArrayNext(coption->params))
+	     cparam = (ppd_cparam_t *)cupsArrayNext(params))
 	{
           switch (cparam->type)
 	  {
@@ -814,6 +939,8 @@ ppdEmitString(ppd_file_t    *ppd,	/* I - PPD file record */
 	        break;
           }
 	}
+
+	cupsArrayDelete(params);
       }
       else
       {
@@ -857,12 +984,24 @@ ppdEmitString(ppd_file_t    *ppd,	/* I - PPD file record */
 }
 
 
+/*
+ * 'ppd_compare_cparams()' - Compare the order of two custom parameters.
+ */
+
+static int				/* O - Result of comparison */
+ppd_compare_cparams(ppd_cparam_t *a,	/* I - First parameter */
+                    ppd_cparam_t *b)	/* I - Second parameter */
+{
+  return (a->order - b->order);
+}
+
+
 /*
  * 'ppd_handle_media()' - Handle media selection...
  */
 
 static void
-ppd_handle_media(ppd_file_t *ppd)
+ppd_handle_media(ppd_file_t *ppd)	/* I - PPD file */
 {
   ppd_choice_t	*manual_feed,		/* ManualFeed choice, if any */
 		*input_slot,		/* InputSlot choice, if any */
@@ -955,5 +1094,5 @@ ppd_sort(ppd_choice_t **c1,	/* I - First choice */
 
 
 /*
- * End of "$Id: emit.c 7224 2008-01-16 23:42:35Z mike $".
+ * End of "$Id: emit.c 7864 2008-08-26 03:43:28Z mike $".
  */
