@@ -1,5 +1,5 @@
 /*
- * "$Id: file.c 6963 2007-09-17 20:35:54Z mike $"
+ * "$Id: file.c 7721 2008-07-11 22:48:49Z mike $"
  *
  *   File functions for the Common UNIX Printing System (CUPS).
  *
@@ -110,7 +110,8 @@ struct _cups_file_s			/**** CUPS file structure... ****/
 		buf[4096],		/* Buffer */
 		*ptr,			/* Pointer into buffer */
 		*end;			/* End of buffer data */
-  off_t		pos;			/* File position for start of buffer */
+  off_t		pos,			/* Position in file */
+		bufpos;			/* File position for start of buffer */
 
 #ifdef HAVE_LIBZ
   z_stream	stream;			/* (De)compression stream */
@@ -402,13 +403,14 @@ cupsFileFlush(cups_file_t *fp)		/* I - CUPS file */
 
   if (!fp || fp->mode != 'w')
   {
-    DEBUG_puts("    Attempt to flush a read-only file...");
+    DEBUG_puts("cupsFileFlush: Attempt to flush a read-only file...");
     return (-1);
   }
 
   bytes = (ssize_t)(fp->ptr - fp->buf);
 
-  DEBUG_printf(("    Flushing %ld bytes...\n", (long)bytes));
+  DEBUG_printf(("cupsFileFlush: Flushing " CUPS_LLFMT " bytes...\n",
+                CUPS_LLCAST bytes));
 
   if (bytes > 0)
   {
@@ -440,6 +442,8 @@ cupsFileGetChar(cups_file_t *fp)	/* I - CUPS file */
   * Range check input...
   */
 
+  DEBUG_printf(("cupsFileGetChar(fp=%p)\n", fp));
+
   if (!fp || (fp->mode != 'r' && fp->mode != 's'))
   {
     DEBUG_puts("cupsFileGetChar: Bad arguments!");
@@ -463,6 +467,10 @@ cupsFileGetChar(cups_file_t *fp)	/* I - CUPS file */
 
   DEBUG_printf(("cupsFileGetChar: Returning %d...\n", *(fp->ptr) & 255));
 
+  fp->pos ++;
+
+  DEBUG_printf(("cupsFileGetChar: pos=" CUPS_LLFMT "\n", CUPS_LLCAST fp->pos));
+
   return (*(fp->ptr)++ & 255);
 }
 
@@ -485,6 +493,10 @@ cupsFileGetConf(cups_file_t *fp,	/* I  - CUPS file */
   * Range check input...
   */
 
+  DEBUG_printf(("cupsFileGetConf(fp=%p, buf=%p, buflen=" CUPS_LLFMT
+                ", value=%p, linenum=%p)\n", fp, buf, CUPS_LLCAST buflen,
+		value, linenum));
+
   if (!fp || (fp->mode != 'r' && fp->mode != 's') ||
       !buf || buflen < 2 || !value)
   {
@@ -623,6 +635,9 @@ cupsFileGetLine(cups_file_t *fp,	/* I - File to read from */
   * Range check input...
   */
 
+  DEBUG_printf(("cupsFileGetLine(fp=%p, buf=%p, buflen=" CUPS_LLFMT ")\n",
+                fp, buf, CUPS_LLCAST buflen));
+
   if (!fp || (fp->mode != 'r' && fp->mode != 's') || !buf || buflen < 3)
     return (0);
 
@@ -637,6 +652,7 @@ cupsFileGetLine(cups_file_t *fp,	/* I - File to read from */
         break;
 
     *ptr++ = ch = *(fp->ptr)++;
+    fp->pos ++;
 
     if (ch == '\r')
     {
@@ -649,7 +665,10 @@ cupsFileGetLine(cups_file_t *fp,	/* I - File to read from */
           break;
 
       if (*(fp->ptr) == '\n')
+      {
         *ptr++ = *(fp->ptr)++;
+	fp->pos ++;
+      }
 
       break;
     }
@@ -665,6 +684,8 @@ cupsFileGetLine(cups_file_t *fp,	/* I - File to read from */
 
   *ptr = '\0';
 
+  DEBUG_printf(("cupsFileGetLine: pos=" CUPS_LLFMT "\n", CUPS_LLCAST fp->pos));
+
   return (ptr - buf);
 }
 
@@ -687,6 +708,9 @@ cupsFileGets(cups_file_t *fp,		/* I - CUPS file */
   * Range check input...
   */
 
+  DEBUG_printf(("cupsFileGets(fp=%p, buf=%p, buflen=" CUPS_LLFMT ")\n", fp, buf,
+                CUPS_LLCAST buflen));
+
   if (!fp || (fp->mode != 'r' && fp->mode != 's') || !buf || buflen < 2)
     return (NULL);
 
@@ -706,6 +730,7 @@ cupsFileGets(cups_file_t *fp,		/* I - CUPS file */
       }
 
     ch = *(fp->ptr)++;
+    fp->pos ++;
 
     if (ch == '\r')
     {
@@ -718,7 +743,10 @@ cupsFileGets(cups_file_t *fp,		/* I - CUPS file */
           break;
 
       if (*(fp->ptr) == '\n')
-        fp->ptr ++;      
+      {
+        fp->ptr ++;
+	fp->pos ++;
+      }
 
       break;
     }
@@ -736,6 +764,8 @@ cupsFileGets(cups_file_t *fp,		/* I - CUPS file */
 
   *ptr = '\0';
 
+  DEBUG_printf(("cupsFileGets: pos=" CUPS_LLFMT "\n", CUPS_LLCAST fp->pos));
+
   return (buf);
 }
 
@@ -801,7 +831,8 @@ cupsFileOpen(const char *filename,	/* I - Name of file */
   */
 
   if (!filename || !mode ||
-      (*mode != 'r' && *mode != 'w' && *mode != 'a' && *mode != 's'))
+      (*mode != 'r' && *mode != 'w' && *mode != 'a' && *mode != 's') ||
+      (*mode == 'a' && isdigit(mode[1] & 255)))
     return (NULL);
 
  /*
@@ -893,7 +924,8 @@ cupsFileOpenFd(int        fd,		/* I - File descriptor */
   */
 
   if (fd < 0 || !mode ||
-      (*mode != 'r' && *mode != 'w' && *mode != 'a' && *mode != 's'))
+      (*mode != 'r' && *mode != 'w' && *mode != 'a' && *mode != 's') ||
+      (*mode == 'a' && isdigit(mode[1] & 255)))
     return (NULL);
 
  /*
@@ -911,8 +943,10 @@ cupsFileOpenFd(int        fd,		/* I - File descriptor */
 
   switch (*mode)
   {
-    case 'w' :
     case 'a' :
+        fp->pos = lseek(fd, 0, SEEK_END);
+
+    case 'w' :
 	fp->mode = 'w';
 	fp->ptr  = fp->buf;
 	fp->end  = fp->buf + sizeof(fp->buf);
@@ -1039,7 +1073,16 @@ cupsFilePrintf(cups_file_t *fp,		/* I - CUPS file */
     return (-1);
 
   if (fp->mode == 's')
-    return (cups_write(fp, buf, bytes));
+  {
+    if (cups_write(fp, buf, bytes) < 0)
+      return (-1);
+
+    fp->pos += bytes;
+
+    DEBUG_printf(("cupsFilePrintf: pos=" CUPS_LLFMT "\n", CUPS_LLCAST fp->pos));
+
+    return (bytes);
+  }
 
   if ((fp->ptr + bytes) > fp->end)
     if (cupsFileFlush(fp))
@@ -1047,6 +1090,8 @@ cupsFilePrintf(cups_file_t *fp,		/* I - CUPS file */
 
   fp->pos += bytes;
 
+  DEBUG_printf(("cupsFilePrintf: pos=" CUPS_LLFMT "\n", CUPS_LLCAST fp->pos));
+
   if (bytes > sizeof(fp->buf))
   {
 #ifdef HAVE_LIBZ
@@ -1109,6 +1154,8 @@ cupsFilePutChar(cups_file_t *fp,	/* I - CUPS file */
 
   fp->pos ++;
 
+  DEBUG_printf(("cupsFilePutChar: pos=" CUPS_LLFMT "\n", CUPS_LLCAST fp->pos));
+
   return (0);
 }
 
@@ -1144,6 +1191,8 @@ cupsFilePuts(cups_file_t *fp,		/* I - CUPS file */
 
     fp->pos += bytes;
 
+    DEBUG_printf(("cupsFilePuts: pos=" CUPS_LLFMT "\n", CUPS_LLCAST fp->pos));
+
     return (bytes);
   }
 
@@ -1153,6 +1202,8 @@ cupsFilePuts(cups_file_t *fp,		/* I - CUPS file */
 
   fp->pos += bytes;
 
+  DEBUG_printf(("cupsFilePuts: pos=" CUPS_LLFMT "\n", CUPS_LLCAST fp->pos));
+
   if (bytes > sizeof(fp->buf))
   {
 #ifdef HAVE_LIBZ
@@ -1184,8 +1235,8 @@ cupsFileRead(cups_file_t *fp,		/* I - CUPS file */
   ssize_t	count;			/* Bytes read */
 
 
-  DEBUG_printf(("cupsFileRead(fp=%p, buf=%p, bytes=%ld)\n", fp, buf,
-                (long)bytes));
+  DEBUG_printf(("cupsFileRead(fp=%p, buf=%p, bytes=" CUPS_LLFMT ")\n", fp, buf,
+                CUPS_LLCAST bytes));
 
  /*
   * Range check input...
@@ -1207,7 +1258,8 @@ cupsFileRead(cups_file_t *fp,		/* I - CUPS file */
     if (fp->ptr >= fp->end)
       if (cups_fill(fp) <= 0)
       {
-        DEBUG_printf(("    cups_fill() returned -1, total=%d\n", (int)total));
+        DEBUG_printf(("cupsFileRead: cups_fill() returned -1, total=" CUPS_LLFMT "\n",
+	              CUPS_LLCAST total));
 
         if (total > 0)
           return ((ssize_t)total);
@@ -1221,6 +1273,9 @@ cupsFileRead(cups_file_t *fp,		/* I - CUPS file */
 
     memcpy(buf, fp->ptr, count);
     fp->ptr += count;
+    fp->pos += count;
+
+    DEBUG_printf(("cupsFileRead: pos=" CUPS_LLFMT "\n", CUPS_LLCAST fp->pos));
 
    /*
     * Update the counts for the last read...
@@ -1235,7 +1290,7 @@ cupsFileRead(cups_file_t *fp,		/* I - CUPS file */
   * Return the total number of bytes read...
   */
 
-  DEBUG_printf(("    total=%d\n", (int)total));
+  DEBUG_printf(("cupsFileRead: total=%d\n", (int)total));
 
   return ((ssize_t)total);
 }
@@ -1252,6 +1307,9 @@ cupsFileRewind(cups_file_t *fp)		/* I - CUPS file */
   * Range check input...
   */
 
+  DEBUG_printf(("cupsFileRewind(fp=%p)\n", fp));
+  DEBUG_printf(("cupsFileRewind: pos=" CUPS_LLFMT "\n", CUPS_LLCAST fp->pos));
+
   if (!fp || fp->mode != 'r')
     return (-1);
 
@@ -1259,18 +1317,22 @@ cupsFileRewind(cups_file_t *fp)		/* I - CUPS file */
   * Handle special cases...
   */
 
-  if (fp->pos == 0)
+  if (fp->bufpos == 0)
   {
    /*
     * No seeking necessary...
     */
 
+    fp->pos = 0;
+
     if (fp->ptr)
     {
       fp->ptr = fp->buf;
       fp->eof = 0;
     }
 
+    DEBUG_printf(("cupsFileRewind: pos=" CUPS_LLFMT "\n", CUPS_LLCAST fp->pos));
+
     return (0);
   }
 
@@ -1288,10 +1350,13 @@ cupsFileRewind(cups_file_t *fp)		/* I - CUPS file */
 
   lseek(fp->fd, 0, SEEK_SET);
 
-  fp->pos = 0;
-  fp->ptr = NULL;
-  fp->end = NULL;
-  fp->eof = 0;
+  fp->bufpos = 0;
+  fp->pos    = 0;
+  fp->ptr    = NULL;
+  fp->end    = NULL;
+  fp->eof    = 0;
+
+  DEBUG_printf(("cupsFileRewind: pos=" CUPS_LLFMT "\n", CUPS_LLCAST fp->pos));
 
   return (0);
 }
@@ -1308,9 +1373,10 @@ cupsFileSeek(cups_file_t *fp,		/* I - CUPS file */
   ssize_t	bytes;			/* Number bytes in buffer */
 
 
-  DEBUG_printf(("cupsFileSeek(fp=%p, pos=" CUPS_LLFMT ")\n", fp, pos));
-  DEBUG_printf(("    fp->pos=" CUPS_LLFMT "\n", fp->pos));
-  DEBUG_printf(("    fp->ptr=%p, fp->end=%p\n", fp->ptr, fp->end));
+  DEBUG_printf(("cupsFileSeek(fp=%p, pos=" CUPS_LLFMT ")\n", fp,
+                CUPS_LLCAST pos));
+  DEBUG_printf(("cupsFileSeek: fp->pos=" CUPS_LLFMT "\n", CUPS_LLCAST fp->pos));
+  DEBUG_printf(("cupsFileSeek: fp->ptr=%p, fp->end=%p\n", fp->ptr, fp->end));
 
  /*
   * Range check input...
@@ -1326,19 +1392,22 @@ cupsFileSeek(cups_file_t *fp,		/* I - CUPS file */
   if (pos == 0)
     return (cupsFileRewind(fp));
 
-  if (fp->pos == pos)
+  if (fp->ptr)
   {
-   /*
-    * No seeking necessary...
-    */
+    bytes = (ssize_t)(fp->end - fp->buf);
 
-    if (fp->ptr)
+    if (pos >= fp->bufpos && pos < (fp->bufpos + bytes))
     {
-      fp->ptr = fp->buf;
-      fp->eof = 0;
-    }
+     /*
+      * No seeking necessary...
+      */
 
-    return (pos);
+      fp->pos = pos;
+      fp->ptr = fp->buf + pos - fp->bufpos;
+      fp->eof = 0;
+
+      return (pos);
+    }
   }
 
 #ifdef HAVE_LIBZ
@@ -1354,26 +1423,20 @@ cupsFileSeek(cups_file_t *fp,		/* I - CUPS file */
 #endif /* HAVE_LIBZ */
 
  /*
-  * Figure out the number of bytes in the current buffer, and then
-  * see if we are outside of it...
+  * Seek forwards or backwards...
   */
 
-  if (fp->ptr)
-    bytes = (ssize_t)(fp->end - fp->buf);
-  else
-    bytes = 0;
-
   fp->eof = 0;
 
-  DEBUG_printf(("    bytes=" CUPS_LLFMT "\n", CUPS_LLCAST bytes));
+  DEBUG_printf(("cupsFileSeek: bytes=" CUPS_LLFMT "\n", CUPS_LLCAST bytes));
 
-  if (pos < fp->pos)
+  if (pos < fp->bufpos)
   {
    /*
     * Need to seek backwards...
     */
 
-    DEBUG_puts("    SEEK BACKWARDS");
+    DEBUG_puts("cupsFileSeek: SEEK BACKWARDS");
 
 #ifdef HAVE_LIBZ
     if (fp->compressed)
@@ -1381,72 +1444,70 @@ cupsFileSeek(cups_file_t *fp,		/* I - CUPS file */
       inflateEnd(&fp->stream);
 
       lseek(fp->fd, 0, SEEK_SET);
-      fp->pos = 0;
-      fp->ptr = NULL;
-      fp->end = NULL;
+      fp->bufpos = 0;
+      fp->pos    = 0;
+      fp->ptr    = NULL;
+      fp->end    = NULL;
 
       while ((bytes = cups_fill(fp)) > 0)
-        if (pos >= fp->pos && pos < (fp->pos + bytes))
+        if (pos >= fp->bufpos && pos < (fp->bufpos + bytes))
 	  break;
 
       if (bytes <= 0)
         return (-1);
 
-      fp->ptr = fp->buf + pos - fp->pos;
+      fp->ptr = fp->buf + pos - fp->bufpos;
+      fp->pos = pos;
     }
     else
 #endif /* HAVE_LIBZ */
     {
-      fp->pos = lseek(fp->fd, pos, SEEK_SET);
-      fp->ptr = NULL;
-      fp->end = NULL;
+      fp->bufpos = lseek(fp->fd, pos, SEEK_SET);
+      fp->pos    = fp->bufpos;
+      fp->ptr    = NULL;
+      fp->end    = NULL;
 
-      DEBUG_printf(("    lseek() returned %ld...\n", (long)fp->pos));
+      DEBUG_printf(("cupsFileSeek: lseek() returned " CUPS_LLFMT "...\n",
+                    CUPS_LLCAST fp->pos));
     }
   }
-  else if (pos >= (fp->pos + bytes))
+  else
   {
    /*
     * Need to seek forwards...
     */
 
-    DEBUG_puts("    SEEK FORWARDS");
+    DEBUG_puts("cupsFileSeek: SEEK FORWARDS");
 
 #ifdef HAVE_LIBZ
     if (fp->compressed)
     {
       while ((bytes = cups_fill(fp)) > 0)
       {
-        if (pos >= fp->pos && pos < (fp->pos + bytes))
+        if (pos >= fp->bufpos && pos < (fp->bufpos + bytes))
 	  break;
       }
 
       if (bytes <= 0)
         return (-1);
 
-      fp->ptr = fp->buf + pos - fp->pos;
+      fp->ptr = fp->buf + pos - fp->bufpos;
+      fp->pos = pos;
     }
     else
 #endif /* HAVE_LIBZ */
     {
-      fp->pos = lseek(fp->fd, pos, SEEK_SET);
-      fp->ptr = NULL;
-      fp->end = NULL;
+      fp->bufpos = lseek(fp->fd, pos, SEEK_SET);
+      fp->pos    = fp->bufpos;
+      fp->ptr    = NULL;
+      fp->end    = NULL;
 
-      DEBUG_printf(("    lseek() returned " CUPS_LLFMT "...\n", fp->pos));
+      DEBUG_printf(("cupsFileSeek: lseek() returned " CUPS_LLFMT "...\n",
+                    CUPS_LLCAST fp->pos));
     }
   }
-  else
-  {
-   /*
-    * Just reposition the current pointer, since we have the right
-    * range...
-    */
 
-    DEBUG_puts("    SEEK INSIDE BUFFER");
-
-    fp->ptr = fp->buf + pos - fp->pos;
-  }
+  DEBUG_printf(("cupsFileSeek: pos=" CUPS_LLFMT "\n", CUPS_LLCAST fp->pos));
 
   return (fp->pos);
 }
@@ -1555,6 +1616,9 @@ cupsFileStdout(void)
 off_t					/* O - File position */
 cupsFileTell(cups_file_t *fp)		/* I - CUPS file */
 {
+  DEBUG_printf(("cupsFileTell(fp=%p)\n", fp));
+  DEBUG_printf(("cupsFileTell: pos=" CUPS_LLFMT "\n", CUPS_LLCAST fp->pos));
+
   return (fp ? fp->pos : 0);
 }
 
@@ -1570,6 +1634,8 @@ cupsFileUnlock(cups_file_t *fp)		/* I - File to lock */
   * Range check...
   */
 
+  DEBUG_printf(("cupsFileUnlock(fp=%p)\n", fp));
+
   if (!fp || fp->mode == 's')
     return (-1);
 
@@ -1598,6 +1664,9 @@ cupsFileWrite(cups_file_t *fp,		/* I - CUPS file */
   * Range check input...
   */
 
+  DEBUG_printf(("cupsFileWrite(fp=%p, buf=%p, bytes=" CUPS_LLFMT ")\n",
+                fp, buf, CUPS_LLCAST bytes));
+
   if (!fp || !buf || bytes < 0 || (fp->mode != 'w' && fp->mode != 's'))
     return (-1);
 
@@ -1615,6 +1684,8 @@ cupsFileWrite(cups_file_t *fp,		/* I - CUPS file */
 
     fp->pos += (off_t)bytes;
 
+    DEBUG_printf(("cupsFileWrite: pos=" CUPS_LLFMT "\n", CUPS_LLCAST fp->pos));
+
     return ((ssize_t)bytes);
   }
 
@@ -1624,6 +1695,8 @@ cupsFileWrite(cups_file_t *fp,		/* I - CUPS file */
 
   fp->pos += (off_t)bytes;
 
+  DEBUG_printf(("cupsFileWrite: pos=" CUPS_LLFMT "\n", CUPS_LLCAST fp->pos));
+
   if (bytes > sizeof(fp->buf))
   {
 #ifdef HAVE_LIBZ
@@ -1652,8 +1725,8 @@ cups_compress(cups_file_t *fp,		/* I - CUPS file */
               const char  *buf,		/* I - Buffer */
 	      size_t      bytes)	/* I - Number bytes */
 {
-  DEBUG_printf(("cups_compress(fp=%p, buf=%p, bytes=%ld)\n", fp, buf,
-                (long)bytes));
+  DEBUG_printf(("cups_compress(fp=%p, buf=%p, bytes=" CUPS_LLFMT "\n", fp, buf,
+                CUPS_LLCAST bytes));
 
  /*
   * Update the CRC...
@@ -1674,8 +1747,8 @@ cups_compress(cups_file_t *fp,		/* I - CUPS file */
     * Flush the current buffer...
     */
 
-    DEBUG_printf(("    avail_in=%d, avail_out=%d\n", fp->stream.avail_in,
-                  fp->stream.avail_out));
+    DEBUG_printf(("cups_compress: avail_in=%d, avail_out=%d\n",
+                  fp->stream.avail_in, fp->stream.avail_out));
 
     if (fp->stream.avail_out < (int)(sizeof(fp->cbuf) / 8))
     {
@@ -1710,19 +1783,15 @@ cups_fill(cups_file_t *fp)		/* I - CUPS file */
 
 
   DEBUG_printf(("cups_fill(fp=%p)\n", fp));
-  DEBUG_printf(("    fp->ptr=%p, fp->end=%p, fp->buf=%p, "
-                "fp->pos=" CUPS_LLFMT ", fp->eof=%d\n",
-                fp->ptr, fp->end, fp->buf, fp->pos, fp->eof));
-
- /*
-  * Update the "pos" element as needed...
-  */
+  DEBUG_printf(("cups_fill: fp->ptr=%p, fp->end=%p, fp->buf=%p, "
+                "fp->bufpos=" CUPS_LLFMT ", fp->eof=%d\n",
+                fp->ptr, fp->end, fp->buf, CUPS_LLCAST fp->bufpos, fp->eof));
 
   if (fp->ptr && fp->end)
-    fp->pos += (off_t)(fp->end - fp->buf);
+    fp->bufpos += fp->end - fp->buf;
 
 #ifdef HAVE_LIBZ
-  DEBUG_printf(("    fp->compressed=%d\n", fp->compressed));
+  DEBUG_printf(("cups_fill: fp->compressed=%d\n", fp->compressed));
 
   while (!fp->ptr || fp->compressed)
   {
@@ -1750,7 +1819,7 @@ cups_fill(cups_file_t *fp)		/* I - CUPS file */
 	* Can't read from file!
 	*/
 
-        DEBUG_printf(("    cups_read() returned " CUPS_LLFMT "!\n",
+        DEBUG_printf(("cups_fill: cups_read() returned " CUPS_LLFMT "!\n",
 	              CUPS_LLCAST bytes));
 
 	return (-1);
@@ -2030,6 +2099,9 @@ cups_read(cups_file_t *fp,		/* I - CUPS file */
   ssize_t	total;			/* Total bytes read */
 
 
+  DEBUG_printf(("cups_read(fp=%p, buf=%p, bytes=" CUPS_LLFMT ")\n", fp, buf,
+                CUPS_LLCAST bytes));
+
  /*
   * Loop until we read at least 0 bytes...
   */
@@ -2048,6 +2120,8 @@ cups_read(cups_file_t *fp,		/* I - CUPS file */
       total = read(fp->fd, buf, bytes);
 #endif /* WIN32 */
 
+    DEBUG_printf(("cups_read: total=" CUPS_LLFMT "\n", CUPS_LLCAST total));
+
     if (total >= 0)
       break;
 
@@ -2082,8 +2156,8 @@ cups_write(cups_file_t *fp,		/* I - CUPS file */
   ssize_t	count;			/* Count this time */
 
 
-  DEBUG_printf(("cups_write(fp=%p, buf=%p, bytes=%ld)\n", fp, buf,
-                (long)bytes));
+  DEBUG_printf(("cups_write(fp=%p, buf=%p, bytes=" CUPS_LLFMT ")\n", fp, buf,
+                CUPS_LLCAST bytes));
 
  /*
   * Loop until all bytes are written...
@@ -2104,6 +2178,8 @@ cups_write(cups_file_t *fp,		/* I - CUPS file */
       count = write(fp->fd, buf, bytes);
 #endif /* WIN32 */
 
+    DEBUG_printf(("cups_write: count=" CUPS_LLFMT "\n", CUPS_LLCAST count));
+
     if (count < 0)
     {
      /*
@@ -2116,8 +2192,6 @@ cups_write(cups_file_t *fp,		/* I - CUPS file */
         return (-1);
     }
 
-    DEBUG_printf(("    count=%ld\n", (long)count));
-
    /*
     * Update the counts for the last write call...
     */
@@ -2136,5 +2210,5 @@ cups_write(cups_file_t *fp,		/* I - CUPS file */
 
 
 /*
- * End of "$Id: file.c 6963 2007-09-17 20:35:54Z mike $".
+ * End of "$Id: file.c 7721 2008-07-11 22:48:49Z mike $".
  */
