@@ -1,5 +1,5 @@
 /*
- * "$Id: http-support.c 7319 2008-02-15 23:26:51Z mike $"
+ * "$Id: http-support.c 7721 2008-07-11 22:48:49Z mike $"
  *
  *   HTTP support routines for the Common UNIX Printing System (CUPS) scheduler.
  *
@@ -36,6 +36,7 @@
  *   httpStatus()         - Return a short string describing a HTTP status code.
  *   _cups_hstrerror()    - hstrerror() emulation function for Solaris and
  *                          others...
+ *   _httpEncodeURI()     - Percent-encode a HTTP request URI.
  *   http_copy_decode()   - Copy and decode a URI.
  *   http_copy_encode()   - Copy and encode a URI.
  */
@@ -1203,6 +1204,20 @@ _cups_hstrerror(int error)		/* I - Error number */
 #endif /* !HAVE_HSTRERROR */
 
 
+/*
+ * '_httpEncodeURI()' - Percent-encode a HTTP request URI.
+ */
+
+char *					/* O - Encoded URI */
+_httpEncodeURI(char       *dst,		/* I - Destination buffer */
+               const char *src,		/* I - Source URI */
+	       size_t     dstsize)	/* I - Size of destination buffer */
+{
+  http_copy_encode(dst, src, dst + dstsize - 1, NULL, NULL, 1);
+  return (dst);
+}
+
+
 /*
  * 'http_copy_decode()' - Copy and decode a URI.
  */
@@ -1311,6 +1326,8 @@ http_copy_encode(char       *dst,	/* O - Destination buffer */
       *dst++ = *src++;
   }
 
+  *dst = '\0';
+
   if (*src)
     return (NULL);
   else
@@ -1319,5 +1336,5 @@ http_copy_encode(char       *dst,	/* O - Destination buffer */
 
 
 /*
- * End of "$Id: http-support.c 7319 2008-02-15 23:26:51Z mike $".
+ * End of "$Id: http-support.c 7721 2008-07-11 22:48:49Z mike $".
  */
