@@ -1,25 +1,16 @@
 /*
- * "$Id: ppd.c 6581 2007-06-20 21:17:49Z mike $"
+ * "$Id: ppd.c 7908 2008-09-03 20:21:01Z mike $"
  *
  *   PPD file routines for the Common UNIX Printing System (CUPS).
  *
+ *   Copyright 2007-2008 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   PostScript is a trademark of Adobe Systems, Inc.
  *
@@ -48,6 +39,8 @@
  *   ppd_add_choice()       - Add a choice to an option.
  *   ppd_add_size()         - Add a page size.
  *   ppd_compare_attrs()    - Compare two attributes.
+ *   ppd_compare_choices()  - Compare two choices...
+ *   ppd_compare_consts()   - Compare two constraints.
  *   ppd_compare_coptions() - Compare two custom options.
  *   ppd_compare_cparams()  - Compare two custom parameters.
  *   ppd_compare_options()  - Compare two options.
@@ -58,6 +51,7 @@
  *   ppd_get_cparam()       - Get a custom parameter record.
  *   ppd_get_group()        - Find or create the named group as needed.
  *   ppd_get_option()       - Find or create the named option as needed.
+ *   ppd_hash_option()      - Generate a hash of the option name...
  *   ppd_read()             - Read a line from a PPD file, skipping comment
  *                            lines as necessary.
  */
@@ -90,6 +84,8 @@
 #define PPD_TEXT	4		/* Line contained human-readable text */
 #define PPD_STRING	8		/* Line contained a string or code */
 
+#define PPD_HASHSIZE	512		/* Size of hash */
+
 
 /*
  * Local functions...
@@ -101,6 +97,8 @@ static ppd_attr_t	*ppd_add_attr(ppd_file_t *ppd, const char *name,
 static ppd_choice_t	*ppd_add_choice(ppd_option_t *option, const char *name);
 static ppd_size_t	*ppd_add_size(ppd_file_t *ppd, const char *name);
 static int		ppd_compare_attrs(ppd_attr_t *a, ppd_attr_t *b);
+static int		ppd_compare_choices(ppd_choice_t *a, ppd_choice_t *b);
+static int		ppd_compare_consts(ppd_const_t *a, ppd_const_t *b);
 static int		ppd_compare_coptions(ppd_coption_t *a,
 			                     ppd_coption_t *b);
 static int		ppd_compare_cparams(ppd_cparam_t *a, ppd_cparam_t *b);
@@ -116,6 +114,7 @@ static ppd_group_t	*ppd_get_group(ppd_file_t *ppd, const char *name,
 			               const char *text, _cups_globals_t *cg,
 				       cups_encoding_t encoding);
 static ppd_option_t	*ppd_get_option(ppd_group_t *group, const char *name);
+static int		ppd_hash_option(ppd_option_t *option);
 static int		ppd_read(cups_file_t *fp, char *keyword, char *option,
 			         char *text, char **string, int ignoreblank,
 				 _cups_globals_t *cg);
@@ -184,6 +183,7 @@ ppdClose(ppd_file_t *ppd)		/* I - PPD file record */
   }
 
   cupsArrayDelete(ppd->options);
+  cupsArrayDelete(ppd->marked);
 
  /*
   * Free any page sizes...
@@ -454,6 +454,9 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
   cups_encoding_t	encoding;	/* Encoding of PPD file */
   _cups_globals_t	*cg = _cupsGlobals();
 					/* Global data */
+  char			custom_name[PPD_MAX_NAME];
+					/* CustomFoo attribute name */
+  ppd_attr_t		*custom_attr;	/* CustomFoo attribute */
   static const char * const ui_keywords[] =
 			{
 #ifdef CUPS_USE_FULL_UI_KEYWORDS_LIST
@@ -573,7 +576,7 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
     return (NULL);
   }
 
-  ppd->language_level = 1;
+  ppd->language_level = 2;
   ppd->color_device   = 0;
   ppd->colorspace     = PPD_CS_GRAY;
   ppd->landscape      = -90;
@@ -620,11 +623,8 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
     puts("");
 #endif /* DEBUG */
 
-    if (strcmp(keyword, "CloseUI") && strcmp(keyword, "CloseGroup") &&
-	strcmp(keyword, "CloseSubGroup") && strncmp(keyword, "Default", 7) &&
-        strcmp(keyword, "JCLCloseUI") && strcmp(keyword, "JCLOpenUI") &&
-	strcmp(keyword, "OpenUI") && strcmp(keyword, "OpenGroup") &&
-	strcmp(keyword, "OpenSubGroup") && string == NULL)
+    if (strncmp(keyword, "Default", 7) && !string &&
+        cg->ppd_conform != PPD_CONFORM_RELAXED)
     {
      /*
       * Need a string value!
@@ -811,6 +811,13 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
         profile = realloc(ppd->profiles, sizeof(ppd_profile_t) *
 	                                 (ppd->num_profiles + 1));
 
+      if (!profile)
+      {
+        cg->ppd_status = PPD_ALLOC_ERROR;
+
+	goto error;
+      }
+
       ppd->profiles     = profile;
       profile           += ppd->num_profiles;
       ppd->num_profiles ++;
@@ -1005,44 +1012,14 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
     }
     else if (!strncmp(keyword, "Custom", 6) && !strcmp(name, "True") && !option)
     {
+      ppd_option_t	*custom_option;	/* Custom option */
+
       DEBUG_puts("Processing Custom option...");
 
      /*
       * Get the option and custom option...
       */
 
-      if ((option = ppdFindOption(ppd, keyword + 6)) == NULL)
-      {
-        int		groupidx;	/* Index for current group */
-	ppd_group_t	*gtemp;		/* Temporary group */
-
-
-        DEBUG_printf(("%s option not found for %s...\n", keyword + 6, keyword));
-
-        if (group)
-          groupidx = group - ppd->groups; /* Save index for current group */
-
-	if ((gtemp = ppd_get_group(ppd, "General", _("General"), cg,
-	                           encoding)) == NULL)
-	{
-	  DEBUG_puts("Unable to get general group!");
-
-	  goto error;
-	}
-
-        if (group)
-          group = ppd->groups + groupidx; /* Restore group pointer */
-
-	if ((option = ppd_get_option(gtemp, keyword + 6)) == NULL)
-	{
-	  DEBUG_printf(("Unable to get %s option!\n", keyword + 6));
-
-          cg->ppd_status = PPD_ALLOC_ERROR;
-
-	  goto error;
-	}
-      }
-
       if (!ppd_get_coption(ppd, keyword + 6))
       {
         cg->ppd_status = PPD_ALLOC_ERROR;
@@ -1050,74 +1027,70 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
 	goto error;
       }
 
-     /*
-      * Add the "custom" option...
-      */
+      if (option && !strcasecmp(option->keyword, keyword + 6))
+        custom_option = option;
+      else
+        custom_option = ppdFindOption(ppd, keyword + 6);
 
-      if ((choice = ppd_add_choice(option, "Custom")) == NULL)
+      if (custom_option)
       {
-	DEBUG_puts("Unable to add Custom choice!");
+       /*
+	* Add the "custom" option...
+	*/
 
-        cg->ppd_status = PPD_ALLOC_ERROR;
+        if ((choice = ppdFindChoice(custom_option, "Custom")) == NULL)
+	  if ((choice = ppd_add_choice(custom_option, "Custom")) == NULL)
+	  {
+	    DEBUG_puts("Unable to add Custom choice!");
 
-	goto error;
+	    cg->ppd_status = PPD_ALLOC_ERROR;
+
+	    goto error;
+	  }
+
+	strlcpy(choice->text, text[0] ? text : _("Custom"),
+		sizeof(choice->text));
+
+	choice->code = strdup(string);
+
+	if (custom_option->section == PPD_ORDER_JCL)
+	  ppd_decode(choice->code);
       }
 
-      strlcpy(choice->text, text[0] ? text : _("Custom"),
-              sizeof(choice->text));
-
-      choice->code = string;
-      string       = NULL;		/* Don't add as an attribute below */
-      option       = NULL;
-
      /*
       * Now process custom page sizes specially...
       */
 
       if (!strcmp(keyword, "CustomPageSize"))
       {
-	ppd->variable_sizes = 1;
-
        /*
 	* Add a "Custom" page size entry...
 	*/
 
+	ppd->variable_sizes = 1;
+
 	ppd_add_size(ppd, "Custom");
 
-	if ((option = ppdFindOption(ppd, "PageRegion")) == NULL)
+	if (option && !strcasecmp(option->keyword, "PageRegion"))
+	  custom_option = option;
+	else
+	  custom_option = ppdFindOption(ppd, "PageRegion");
+
+        if (custom_option)
 	{
-	  int		groupidx;	/* Index to current group */
-	  ppd_group_t	*gtemp;		/* Temporary group */
+	  if ((choice = ppdFindChoice(custom_option, "Custom")) == NULL)
+	    if ((choice = ppd_add_choice(custom_option, "Custom")) == NULL)
+	    {
+	      DEBUG_puts("Unable to add Custom choice!");
 
-          if (group)
-            groupidx = group - ppd->groups; /* Save index for current group */
+	      cg->ppd_status = PPD_ALLOC_ERROR;
 
-	  if ((gtemp = ppd_get_group(ppd, "General", _("General"), cg,
-				     encoding)) == NULL)
-	  {
-	    DEBUG_puts("Unable to get general group!");
+	      goto error;
+	    }
 
-	    goto error;
-	  }
-
-          if (group)
-            group = ppd->groups + groupidx; /* Restore group pointer */
-
-	  option = ppd_get_option(gtemp, "PageRegion");
+	  strlcpy(choice->text, text[0] ? text : _("Custom"),
+		  sizeof(choice->text));
         }
-
-	if ((choice = ppd_add_choice(option, "Custom")) == NULL)
-	{
-	  DEBUG_puts("Unable to add Custom choice!");
-
-	  cg->ppd_status = PPD_ALLOC_ERROR;
-
-	  goto error;
-	}
-
-	strlcpy(choice->text, text[0] ? text : _("Custom"),
-		sizeof(choice->text));
-        option = NULL;
       }
     }
     else if (!strcmp(keyword, "LandscapeOrientation"))
@@ -1127,7 +1100,7 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
       else if (!strcmp(string, "Plus90"))
         ppd->landscape = 90;
     }
-    else if (!strcmp(keyword, "Emulators"))
+    else if (!strcmp(keyword, "Emulators") && string)
     {
       for (count = 1, sptr = string; sptr != NULL;)
         if ((sptr = strchr(sptr, ' ')) != NULL)
@@ -1138,7 +1111,12 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
 	}
 
       ppd->num_emulations = count;
-      ppd->emulations     = calloc(count, sizeof(ppd_emul_t));
+      if ((ppd->emulations = calloc(count, sizeof(ppd_emul_t))) == NULL)
+      {
+        cg->ppd_status = PPD_ALLOC_ERROR;
+
+	goto error;
+      }
 
       for (i = 0, sptr = string; i < count; i ++)
       {
@@ -1213,6 +1191,8 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
       * Add an option record to the current sub-group, group, or file...
       */
 
+      DEBUG_printf(("name=\"%s\" (%d)\n", name, strlen(name)));
+
       if (name[0] == '*')
         _cups_strcpy(name, name + 1); /* Eliminate leading asterisk */
 
@@ -1298,6 +1278,34 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
 
       ppd_free(string);
       string = NULL;
+
+     /*
+      * Add a custom option choice if we have already seen a CustomFoo
+      * attribute...
+      */
+
+      if (!strcasecmp(name, "PageRegion"))
+        strcpy(custom_name, "CustomPageSize");
+      else
+        snprintf(custom_name, sizeof(custom_name), "Custom%s", name);
+
+      if ((custom_attr = ppdFindAttr(ppd, custom_name, "True")) != NULL)
+      {
+        if ((choice = ppdFindChoice(option, "Custom")) == NULL)
+	  if ((choice = ppd_add_choice(option, "Custom")) == NULL)
+	  {
+	    DEBUG_puts("Unable to add Custom choice!");
+
+	    cg->ppd_status = PPD_ALLOC_ERROR;
+
+	    goto error;
+	  }
+
+	strlcpy(choice->text,
+	        custom_attr->text[0] ? custom_attr->text : _("Custom"),
+		sizeof(choice->text));
+        choice->code = strdup(custom_attr->value);
+      }
     }
     else if (!strcmp(keyword, "JCLOpenUI"))
     {
@@ -1377,6 +1385,30 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
 
       ppd_free(string);
       string = NULL;
+
+     /*
+      * Add a custom option choice if we have already seen a CustomFoo
+      * attribute...
+      */
+
+      snprintf(custom_name, sizeof(custom_name), "Custom%s", name);
+
+      if ((custom_attr = ppdFindAttr(ppd, custom_name, "True")) != NULL)
+      {
+	if ((choice = ppd_add_choice(option, "Custom")) == NULL)
+	{
+	  DEBUG_puts("Unable to add Custom choice!");
+
+	  cg->ppd_status = PPD_ALLOC_ERROR;
+
+	  goto error;
+	}
+
+	strlcpy(choice->text,
+	        custom_attr->text[0] ? custom_attr->text : _("Custom"),
+		sizeof(choice->text));
+        choice->code = strdup(custom_attr->value);
+      }
     }
     else if (!strcmp(keyword, "CloseUI") || !strcmp(keyword, "JCLCloseUI"))
     {
@@ -1849,7 +1881,12 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
       * Add the option choice...
       */
 
-      choice = ppd_add_choice(option, name);
+      if ((choice = ppd_add_choice(option, name)) == NULL)
+      {
+        cg->ppd_status = PPD_ALLOC_ERROR;
+
+	goto error;
+      }
 
       if (text[0])
         cupsCharsetToUTF8((cups_utf8_t *)choice->text, text,
@@ -1886,8 +1923,8 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
   cupsLangFree(language);
 
 #ifdef DEBUG
-  if (!feof(fp))
-    printf("Premature EOF at %lu...\n", (unsigned long)ftell(fp));
+  if (!cupsFileEOF(fp))
+    printf("Premature EOF at %lu...\n", (unsigned long)cupsFileTell(fp));
 #endif /* DEBUG */
 
   if (cg->ppd_status != PPD_OK)
@@ -1906,7 +1943,9 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
   * each choice and custom option...
   */
 
-  ppd->options = cupsArrayNew((cups_array_func_t)ppd_compare_options, NULL);
+  ppd->options = cupsArrayNew2((cups_array_func_t)ppd_compare_options, NULL,
+                               (cups_ahash_func_t)ppd_hash_option,
+			       PPD_HASHSIZE);
 
   for (i = ppd->num_groups, group = ppd->groups;
        i > 0;
@@ -1929,6 +1968,20 @@ ppdOpen2(cups_file_t *fp)		/* I - File to read from */
     }
   }
 
+ /*
+  * Sort the constraints...
+  */
+
+  if (ppd->num_consts > 1)
+    qsort(ppd->consts, ppd->num_consts, sizeof(ppd_const_t),
+          (int (*)(const void *, const void *))ppd_compare_consts);
+
+ /*
+  * Create an array to track the marked choices...
+  */
+
+  ppd->marked = cupsArrayNew((cups_array_func_t)ppd_compare_choices, NULL);
+
  /*
   * Return the PPD file structure...
   */
@@ -2222,6 +2275,40 @@ ppd_compare_attrs(ppd_attr_t *a,	/* I - First attribute */
 }
 
 
+/*
+ * 'ppd_compare_choices()' - Compare two choices...
+ */
+
+static int				/* O - Result of comparison */
+ppd_compare_choices(ppd_choice_t *a,	/* I - First choice */
+                    ppd_choice_t *b)	/* I - Second choice */
+{
+  return (strcmp(a->option->keyword, b->option->keyword));
+}
+
+
+/*
+ * 'ppd_compare_consts()' - Compare two constraints.
+ */
+
+static int				/* O - Result of comparison */
+ppd_compare_consts(ppd_const_t *a,	/* I - First constraint */
+                   ppd_const_t *b)	/* I - Second constraint */
+{
+  int	ret;				/* Result of comparison */
+
+
+  if ((ret = strcmp(a->option1, b->option1)) != 0)
+    return (ret);
+  else if ((ret = strcmp(a->choice1, b->choice1)) != 0)
+    return (ret);
+  else if ((ret = strcmp(a->option2, b->option2)) != 0)
+    return (ret);
+  else
+    return (strcmp(a->choice2, b->choice2));
+}
+
+
 /*
  * 'ppd_compare_coptions()' - Compare two custom options.
  */
@@ -2559,6 +2646,24 @@ ppd_get_option(ppd_group_t *group,	/* I - Group */
 }
 
 
+/*
+ * 'ppd_hash_option()' - Generate a hash of the option name...
+ */
+
+static int				/* O - Hash index */
+ppd_hash_option(ppd_option_t *option)	/* I - Option */
+{
+  int		hash = 0;		/* Hash index */
+  const char	*k;			/* Pointer into keyword */
+
+
+  for (hash = option->keyword[0], k = option->keyword + 1; *k;)
+    hash = 33 * hash + *k++;
+
+  return (hash & 511);
+}
+
+
 /*
  * 'ppd_read()' - Read a line from a PPD file, skipping comment lines as
  *                necessary.
@@ -2763,8 +2868,6 @@ ppd_read(cups_file_t    *fp,		/* I - File to read from */
 	    if (ch == 0x0a)
 	      cupsFileGetChar(fp);
 	  }
-
-	  ch = '\n';
 	}
 	else if (ch < ' ' && ch != '\t' && cg->ppd_conform == PPD_CONFORM_STRICT)
 	{
@@ -3116,5 +3219,5 @@ ppd_read(cups_file_t    *fp,		/* I - File to read from */
 
 
 /*
- * End of "$Id: ppd.c 6581 2007-06-20 21:17:49Z mike $".
+ * End of "$Id: ppd.c 7908 2008-09-03 20:21:01Z mike $".
  */
