@@ -1,10 +1,10 @@
 #!/bin/sh
 #
-# "$Id: 5.5-lp.sh 6649 2007-07-11 21:46:42Z mike $"
+# "$Id: 5.5-lp.sh 7416 2008-03-31 22:33:33Z mike $"
 #
 #   Test the lp command.
 #
-#   Copyright 2007 by Apple Inc.
+#   Copyright 2007-2008 by Apple Inc.
 #   Copyright 1997-2005 by Easy Software Products, all rights reserved.
 #
 #   These coded instructions, statements, and computer programs are the
@@ -16,8 +16,8 @@
 
 echo "LP Default Test"
 echo ""
-echo "    lp testfile.jpg"
-../systemv/lp testfile.jpg 2>&1
+echo "    lp testfile.pdf"
+../systemv/lp testfile.pdf 2>&1
 if test $? != 0; then
 	echo "    FAILED"
 	exit 1
@@ -28,8 +28,8 @@ echo ""
 
 echo "LP Destination Test"
 echo ""
-echo "    lp -d Test1 testfile.jpg"
-../systemv/lp -d Test1 -o job-hold-until=indefinite testfile.jpg 2>&1
+echo "    lp -d Test2 testfile.jpg"
+../systemv/lp -d Test2 testfile.jpg 2>&1
 if test $? != 0; then
 	echo "    FAILED"
 	exit 1
@@ -38,13 +38,27 @@ else
 fi
 echo ""
 
-echo "LP Flood Test"
+echo "LP Options Test"
 echo ""
-echo "    lp -d Test1 testfile.jpg ($1 times in parallel)"
+echo "    lp -d Test1 -P 1-4 -o job-sheets=classified,classified testfile.pdf"
+../systemv/lp -d Test1 -P 1-4 -o job-sheets=classified,classified testfile.pdf 2>&1
+if test $? != 0; then
+	echo "    FAILED"
+	exit 1
+else
+	echo "    PASSED"
+fi
+echo ""
+
+echo "LP Flood Test ($1 times in parallel)"
+echo ""
+echo "    lp -d Test1 testfile.jpg"
+echo "    lp -d Test2 testfile.jpg"
 i=0
 while test $i -lt $1; do
 	echo "    flood copy $i..." 1>&2
 	../systemv/lp -d Test1 testfile.jpg 2>&1 &
+	../systemv/lp -d Test2 testfile.jpg 2>&1 &
 	lppid=$!
 	i=`expr $i + 1`
 done
@@ -57,6 +71,8 @@ else
 fi
 echo ""
 
+./waitjobs.sh
+
 #
-# End of "$Id: 5.5-lp.sh 6649 2007-07-11 21:46:42Z mike $".
+# End of "$Id: 5.5-lp.sh 7416 2008-03-31 22:33:33Z mike $".
 #
