@@ -1,25 +1,16 @@
 /*
- * "$Id: ipptest.c 5878 2006-08-24 15:55:42Z mike $"
+ * "$Id: ipptest.c 7721 2008-07-11 22:48:49Z mike $"
  *
  *   IPP test command for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2006 by Easy Software Products.
+ *   Copyright 2007-2008 by Apple Inc.
+ *   Copyright 1997-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  * Contents:
  *
@@ -142,6 +133,15 @@ main(int  argc,				/* I - Number of command-line arguments */
     {
       if (!strcmp(argv[i], "-v"))
         Verbosity ++;
+      else if (!strcmp(argv[i], "-d"))
+      {
+        i ++;
+
+	if (i >= argc)
+	  usage(NULL);
+	else
+	  putenv(argv[i]);
+      }
       else if (!strcmp(argv[i], "-i"))
       {
         i++;
@@ -268,6 +268,7 @@ do_tests(const char *uri,		/* I - URI to connect on */
   {
     printf("Unable to connect to %s on port %d - %s\n", server, port,
            strerror(errno));
+    fclose(fp);
     return (0);
   }
 
@@ -307,7 +308,6 @@ do_tests(const char *uri,		/* I - URI to connect on */
     request       = ippNew();
     op            = (ipp_op_t)0;
     group         = IPP_TAG_ZERO;
-    value         = IPP_TAG_ZERO;
     num_statuses  = 0;
     num_expects   = 0;
     num_displayed = 0;
@@ -454,9 +454,28 @@ do_tests(const char *uri,		/* I - URI to connect on */
 	      strlcpy(tokenptr, cupsUser(), sizeof(token) - (tokenptr - token));
 	      tempptr += 5;
 	    }
+	    else if (!strncasecmp(tempptr + 1, "ENV[", 4))
+	    {
+	      char *end;		/* End of $ENV[name] */
+
+
+	      if ((end = strchr(tempptr + 5, ']')) != NULL)
+	      {
+	        *end++ = '\0';
+		strlcpy(tokenptr,
+		        getenv(tempptr + 5) ? getenv(tempptr + 5) : tempptr + 5,
+		        sizeof(token) - (tokenptr - token));
+		tempptr = end;
+	      }
+	      else
+	      {
+		*tokenptr++ = *tempptr++;
+		*tokenptr   = '\0';
+	      }
+	    }
             else
 	    {
-	      *tokenptr++ = *tempptr ++;
+	      *tokenptr++ = *tempptr++;
 	      *tokenptr   = '\0';
 	    }
 
@@ -593,7 +612,8 @@ do_tests(const char *uri,		/* I - URI to connect on */
     if (filename[0])
       response = cupsDoFileRequest(http, request, resource, filename);
     else
-      response = cupsDoRequest(http, request, resource);
+      response = cupsDoIORequest(http, request, resource, -1,
+                                 Verbosity ? 1 : -1);
 
     if (response == NULL)
     {
@@ -671,7 +691,7 @@ do_tests(const char *uri,		/* I - URI to connect on */
 	  puts("        BAD STATUS");
 
 	printf("        status-code = %04x (%s)\n",
-	       cupsLastError(), cupsLastErrorString());
+	       cupsLastError(), ippErrorString(cupsLastError()));
 
         for (i = 0; i < num_expects; i ++)
 	  if (ippFindAttribute(response, expects[i], IPP_TAG_ZERO) == NULL)
@@ -924,5 +944,5 @@ usage(const char *option)		/* I - Option string or NULL */
 
 
 /*
- * End of "$Id: ipptest.c 5878 2006-08-24 15:55:42Z mike $".
+ * End of "$Id: ipptest.c 7721 2008-07-11 22:48:49Z mike $".
  */
